require "import"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"

function destroyRecognizer(recognizer)
  if recognizer then
    recognizer.destroy()
    recognizer = nil
  end
  return true
end

function startSpeechRecognition()
  local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)

  local recognitionListener = RecognitionListener {
    onResults = function(results)
      local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
      if resultArray ~= nil and resultArray.size() > 0 then
        local recognizedText = resultArray.get(0) -- Original recognized text

        -- Perform word replacement and text transformations
        local replacedText = replaceWords(recognizedText)

        local editText = service.getEditText()
        service.insertText(editText, replacedText .. " ") -- Insert the text obtained after replacement
        service.speak(replacedText)
      end
      destroyRecognizer(speechRecognizer)
    end,
    onError = function()
      destroyRecognizer(speechRecognizer)
      return false
    end
  }

  local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault())
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
  speechRecognizer.startListening(recognizerIntent)
  speechRecognizer.setRecognitionListener(recognitionListener)

  return true
end

function replaceWords(text)
  local wordReplacements = {
    ["my full name"] = "Hemant Singal",
    ["my first name"] = "Hemant",
    ["my last name"] = "Singal",
    ["my email id"] = "logicallycritical1@gmail.com",
    ["my mobile number"] = "+91098123 98123",
    ["my state name"] = "California",
    ["my city name"] = "Washington DC",
    ["my full address"] = "House  No.10, Silicon Balley Street, Washington DC, USA",
    ["my youtube channel"] = "CSR With MK",
    ["my qualification"] = "B. Tech, Computer Science",




    ["new line"] = "\n",
    ["new paragraph"] = "\n\n",
    ["question symbol"] = "?",
    ["questions symbol"] = "?",
    ["fullstop"] = ".",
    ["full stop"] = ".",
    ["fold stop"] = ".",
    ["food stop"] = ".",
    ["comma"] = ",",
    ["coma"] = ",",
    ["kama"] = ",",
    ["colon"] = ":",
    ["colen"] = ":",
    ["colan"] = ":",
    ["bang symbol"] = "!",
    ["bank symbol"] = "!",
    ["at symbol"] = "@",
    ["ate symbol"] = "@",
    ["hash symbol"] = "#",
    ["has symbol"] = "#",
    ["star symbol"] = "*",
    ["dollar symbol"] = "$",
    ["rupee symbol"] = "₹",
    ["rupees symbol"] = "₹",
    ["rupi symbol"] = "₹",
    ["backflash"] = "\\",
    ["back flash"] = "\\",
    ["more than symbol"] = ">",
    ["mordhan symbol"] = ">",
    ["less than symbol"] = "<",
    ["space bar"] = " ",
    ["plus symbol"] = "+"
  }

  for originalWord, replacementWord in pairs(wordReplacements) do
    -- Create a case-insensitive pattern for whole word matching
    local pattern = originalWord:gsub(
      "(%a)",
      function(char)
        return "[" .. char:lower() .. char:upper() .. "]"
      end
    )

    -- Replace using the case-insensitive pattern
    text = text:gsub(pattern, replacementWord)
  end

  -- Capitalize the first letter of the text
  text = text:gsub("^%l", string.upper)

  -- Capitalize the first letter after recognized punctuation marks or symbols
  text = text:gsub("[!?.:;][%s]*%l", string.upper)

  -- Add a period at the end if there is no specified punctuation and there is at least one visible character
  if not text:match("[,!?.:;]%s*$") and text:match("%S") then
    text = text:gsub("(%S)%s*$", "%1.") -- Add period immediately after the last visible character
  end

  -- Remove spaces before and after specific symbols
  text = removeSpacesBeforeAfterSymbols(text)

  return text
end

function removeSpacesBeforeAfterSymbols(text)
  -- Remove 1 space before specific symbols
  text = text:gsub("(%s)([?.@#*\\>!:,])", "%2")

  -- Remove 1 space after specific symbols
  text = text:gsub("([@#*\\<%$₹])(%s)", "%1")

  return text
end

startSpeechRecognition()
